# -*- coding: utf-8; indent-tabs-mode: nil -*-

# mailbridge: a mail storage spanning bridge
# 
# Copyright 2011 Ryan Hughes <ryan@n-h.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from mailbridge.store.Change import ReadState
from mailbridge.store.Folder import Folder
from mailbridge.util.imapUTF7 import *

import traceback
import sys
import logging


class DsyncExportChangesContents(object):
    '''
    classdocs
    '''


    def __init__(self, folder):
        '''
        Constructor
        '''
        self.folder = folder
        self.current_modseq = None
        self.current_modseq_uid = None
        
    def Config(self, importer, syncstate, flags):
        self.importer = importer
        fromversion = None
        self.folder.getChanges(fromversion)  # TODO: fromversion?
        redis = self.folder.store.redis
        changeids = redis.hkeys(self.folder.get_redis_basekey()+":changes")
        
    def Synchronize(self, progress):
        redis = self.folder.store.redis
        if (redis.llen(self.folder.get_redis_basekey()+":changeidqueue") > 0):
            changeid = redis.lindex(self.folder.get_redis_basekey()+":changeidqueue",0)
            
            change = redis.hget(self.folder.get_redis_basekey()+":changes",changeid)
            modseq,uid = changeid.split(':',1)
            if (change[0] == '1'):
                # new message create msg
                if redis.hget(self.folder.get_redis_basekey()+":uid2sourceid",uid) == None:
                    # not yet mapped -> retrieve message
                    message = self.folder.getMessage(uid)
                    source_key = self.importer.importMessageChange(uid, message)
                    if source_key != None:
                        # save uid to sourceid mapping
                        redis.hset(self.folder.get_redis_basekey()+":uid2sourceid",uid,source_key)
                        # save sourceid to uid mapping
                        redis.hset(self.folder.get_redis_basekey()+":sourceid2uid",source_key,uid)
            elif (change[0] == '2'):
                flagchange,flags,msg_last_seen_flags = change.split(':',2)
                source_key = redis.hget(self.folder.get_redis_basekey()+":uid2sourceid",uid)
                if (source_key != None):
                    message_flags = self.folder.getMessageFlagsFromFlags(flags)
                    message_last_seen_flags = self.folder.getMessageFlagsFromFlags(msg_last_seen_flags)
                    if ((message_flags.flag_answered  != message_last_seen_flags.flag_answered ) or
                        (message_flags.flag_flagged   != message_last_seen_flags.flag_flagged  ) or
                        (message_flags.flag_forwarded != message_last_seen_flags.flag_forwarded) 
                        ):
                        self.importer.importMessageChange(source_key, message_flags)
                        
                    if (message_flags.flag_read != message_last_seen_flags.flag_read) :
                        # read flag changed
                        readstates = []
                        readstate = ReadState(source_key, message_flags.flag_read)
                        readstates.append(readstate)
                        try:
                            self.importer.importMessageReadStateChange(readstates)
                        except:
                            traceback.print_exc(file=sys.stdout)
            elif (change[0] == '3'):
                source_key = redis.hget(self.folder.get_redis_basekey()+":uid2sourceid",uid)
                if (source_key != None):
                    self.importer.importMessageDeletion(1,[source_key])
            elif (change[0] == '4'):
                source_key = redis.hget(self.folder.get_redis_basekey()+":uid2sourceid",uid)
                if (source_key != None):
                    # only soft delete even if IMAP has expunged
                    self.importer.importMessageDeletion(1,[source_key])
            else:
                return (1,1)
                
            changeidpopped = redis.lpop(self.folder.get_redis_basekey()+":changeidqueue")
        else:
            # quit
            return (1,1)

        return (1,2)
        source_key = self.importer.importMessageChange(binascii.hexlify(source_key.Value), msg)
        self.importer.importMessageDeletion(hexentries)
        self.importer.importMessageReadStateChange(readstates)
    
    
class DsyncExportChangesHierarchy(object):
    '''
    classdocs
    '''


    def __init__(self, store):
        '''
        Constructor
        '''
        self.store = store
        self.current_modseq = None
        self.current_modseq_uid = None
        self.changes = 0
        self.logger = logging.getLogger("DsyncExportChangesHierarchy")
        
    def Config(self, importer, syncstate, flags):
        self.importer = importer
        fromversion = None
        self.store.getChanges(fromversion)
        redis = self.store.redis
        changeids = redis.hkeys(self.store.get_redis_basekey()+":changes")
        
    def Synchronize(self, progress):
        # push the folder changes
        redis = self.store.redis
        if (redis.llen(self.store.get_redis_basekey()+":changeidqueue") > 0):
            changeid = redis.lindex(self.store.get_redis_basekey()+":changeidqueue",0)
            
            change = redis.hget(self.store.get_redis_basekey()+":changes",changeid)
            guid,changetype,lastchange = changeid.split(':',2)
            if (changetype == '1'):
                # new folder create folder
                parent_guid,imap_folder_name = change.split(':', 1)
                if (len(parent_guid) > 0):
                    parent_source_key = redis.hget(self.store.get_redis_basekey()+":guid2sourceid", parent_guid)
                else:
                    parent_source_key = ''
                if (parent_source_key != None):
                    if (redis.hget(self.store.get_redis_basekey()+":guid2sourceid", guid) == None):
                        # not yet mapped -> create folder
                        folder = Folder()
                        folder.parent = parent_source_key
                        folder.name = u'' + imapUTF7Decode(imap_folder_name)
                        source_key = self.importer.importFolderChange(None, folder)
                        self.logger.debug("Created Folder Guid %s SourceKey: %s" % (guid, repr(source_key)))
                        if source_key != None:
                            # save guid to sourceid and path mapping
                            self.store.setFolderPathMapping(guid, imap_folder_name, source_key)
            elif (changetype == '2'):
                # move and/or rename folder
                parent_guid,imap_folder_name = change.split(':', 1)
                if (len(parent_guid) > 0):
                    parent_source_key = redis.hget(self.store.get_redis_basekey()+":guid2sourceid", parent_guid)
                else:
                    parent_source_key = ''
                if (parent_source_key != None):
                    source_key = redis.hget(self.store.get_redis_basekey()+":guid2sourceid", guid)
                    if (source_key != None):
                        # not yet mapped -> create folder
                        folder = Folder()
                        folder.id = source_key
                        folder.parent = parent_source_key
                        folder.name = u'' + imapUTF7Decode(imap_folder_name)
                        result = self.importer.importFolderChange(source_key, folder)
            elif (changetype == '3'):
                source_key = redis.hget(self.store.get_redis_basekey()+":guid2sourceid", guid)
                if (source_key != None):
                    self.importer.importFolderDeletion(1,[source_key])
            else:
                return (1,1)
                
            changeidpopped = redis.lpop(self.store.get_redis_basekey()+":changeidqueue")
            self.changes = self.changes + 1
        else:
            # quit
            return (1,1)

        return (1,2)
