# -*- coding: utf-8; indent-tabs-mode: nil -*-

# mailbridge: a mail storage spanning bridge
# 
# Copyright 2011 Ryan Hughes <ryan@n-h.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import logging

import MAPI
from MAPI.Util import *
from MAPI.Time import *
from MAPI.Struct import *


from mailbridge.store.zarafa.ZarafaStore import ZarafaStore


class NoSuchStoreError(Exception):
    def __init__(self): pass


class ZarafaServer(object):
    '''
    classdocs
    '''


    def __init__(self, server = None, **keywords):
        '''
        Constructor
        '''
        if server is None:
            server = os.getenv('ZARAFA_SOCKET', 'https://127.0.0.1:237/zarafa')
        
        self.session = OpenECSession('SYSTEM', '', server, sslkey_file=keywords['sslkey_file'], sslkey_pass=keywords['sslkey_pass'], flags = 0)
        self.store = GetDefaultStore(self.session);
        self.server_guid = self.store.GetProps([0x67c20102], 0)[0].Value
        self.logger = logging.getLogger("ZarafaServer[%s]" % server.replace('.', '_'))

    def get_store(self, user):
        ema = self.store.QueryInterface(IID_IExchangeManageStore)
        try:
            if user == '__public__':
                storeEid = GetPublicStore(self.session)
            else:
                storeEid = ema.CreateStoreEntryID(None, user, 0)
            store = self.session.OpenMsgStore(0, storeEid, IID_IMsgStore, MAPI_BEST_ACCESS)
            # TODO: error handling
            return ZarafaStore(self.session, store)
            
        except MAPIError, err:
            if err.hr == MAPI_E_NOT_FOUND:
                self.logger.debug("Store for user '%s' not found" % user)
                raise NoSuchStoreError()
            self.logger.error("Failed to open store for user '%s'" % user)
            raise

