# -*- coding: utf-8; indent-tabs-mode: nil -*-

# mailbridge: a mail storage spanning bridge
# 
# Copyright 2011 Ryan Hughes <ryan@n-h.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License, version 3,
# as published by the Free Software Foundation
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import MAPI
import MAPICore
import os
import sys
import time
import locale
import binascii
from MAPI.Util import *
from MAPI.Time import *
from MAPI.Struct import *


from mailbridge.store.zarafa.ZarafaStoreImporters import ZarafaStoreContentsImporter
from mailbridge.store.zarafa.ZarafaStoreImporters import ZarafaStoreHierarchyImporter
import mailbridge.store.BaseStore


class Folder(object):
    def __init__(self, id, display_name=None):
        self.id = id
        self.display_name = display_name



class ZarafaStore(mailbridge.store.BaseStore.BaseStore):
    '''
    classdocs
    '''


    def __init__(self, session=None, store=None):
        '''
        Constructor
        '''
        self.session = session
        self.store = store
        if self.store != None:
            self._connected()

    def connect(self, username, password):
        self.session = OpenECSession(username, password, os.getenv('ZARAFA_SOCKET', 'file:///var/run/zarafa'), flags=0)
        #try:
        self.store = GetDefaultStore(self.session)
        #except:
        #    return
        self._connected()
        
    def _connected(self):
        self.store_entryid = self.store.GetProps([PR_ENTRYID], 0)[0].Value
        self.root = self.store.OpenEntry(None, IID_IMAPIFolder, MAPI_BEST_ACCESS|MAPI_DEFERRED_ERRORS)
        self.root_sourcekey = self.root.GetProps([PR_SOURCE_KEY], 0)[0].Value
        self.root_entryid = self.root.GetProps([PR_ENTRYID], 0)[0].Value
        ipm_root_entryid = self.store.GetProps([PR_IPM_SUBTREE_ENTRYID], 0)[0].Value
        ipm_root = self.store.OpenEntry(ipm_root_entryid, None, MAPI_MODIFY)
        self.ipm_root_sourcekey = ipm_root.GetProps([PR_SOURCE_KEY], 0)[0].Value
        self.addressbook = self.session.OpenAddressBook(0, None, 0)
        self.special_source_keys = {}
        folderid = self.getReceiveFolderId()
        folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
        self.special_source_keys[folder.GetProps([PR_SOURCE_KEY], 0)[0].Value] = 1
        folderid = self.getWastebasketFolderId()
        folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
        self.special_source_keys[folder.GetProps([PR_SOURCE_KEY], 0)[0].Value] = 1
        folderid = self.getDraftsFolderId()
        folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
        self.special_source_keys[folder.GetProps([PR_SOURCE_KEY], 0)[0].Value] = 1
        folderid = self.getSentFolderId()
        folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
        self.special_source_keys[folder.GetProps([PR_SOURCE_KEY], 0)[0].Value] = 1
        
    def getReceiveFolder(self):
        eid = self.store.GetReceiveFolder(None, 0)
        folder = self.store.OpenEntry(eid[0], None, MAPI_MODIFY)
        return folder

    def getReceiveFolderId(self):
        eid = self.store.GetReceiveFolder(None, 0)
        return eid[0]

    def getWastebasketFolderId(self):
        return self.store.GetProps([PR_IPM_WASTEBASKET_ENTRYID], 0)[0].Value

    def getDraftsFolderId(self):
        return self.getReceiveFolder().GetProps([PR_IPM_DRAFTS_ENTRYID], 0)[0].Value

    def getSentFolderId(self):
        return self.store.GetProps([PR_IPM_SENTMAIL_ENTRYID], 0)[0].Value

    def getCacheFolderId(self, folderid, cache_store):
        if (folderid == self.getReceiveFolderId()):
            return cache_store.getReceiveFolderId()
        elif (folderid == self.getWastebasketFolderId()):
            return cache_store.getWastebasketFolderId()
        elif (folderid == self.getDraftsFolderId()):
            return cache_store.getDraftsFolderId()
        elif (folderid == self.getSentFolderId()):
            return cache_store.getSentFolderId()
        else:
            folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
            hex_source_key = binascii.hexlify(folder.GetProps([PR_SOURCE_KEY], 0)[0].Value)
            guid = cache_store.getGUIDForSourceKey(hex_source_key)
            if guid != None:
                return guid
            hex_record_key = binascii.hexlify(folder.GetProps([PR_RECORD_KEY], 0)[0].Value)
            return '0' * (32 - len(hex_record_key)) + hex_record_key
            
    def getHierarchy(self):
        folders = []
        ReceiveFolderId = self.getReceiveFolderId()
        WastebasketFolderId = self.getWastebasketFolderId()
        table = self.root.GetHierarchyTable(CONVENIENT_DEPTH)
        table.SetColumns([PR_ENTRYID, PR_FOLDER_TYPE, PR_CONTAINER_CLASS_A, PR_DISPLAY_NAME], TBL_BATCH)
        rows = table.QueryRows(10000, 0)
        for row in rows:
            if row[1].Value != FOLDER_GENERIC:
                continue
            container_class = row[2]
            if row[0].Value in [ReceiveFolderId, WastebasketFolderId] or str(container_class.Value) == 'IPF.Note' or PROP_TYPE(container_class.ulPropTag)==MAPICore.PT_ERROR:
                folder = Folder(row[0].Value)
                folders.append(folder)
        return folders
        
    def getExporter(self, folderid=None, folder=None):
        if folder:
            exporter = folder.OpenProperty(PR_CONTENTS_SYNCHRONIZER, IID_IExchangeExportChanges, 0, 0)
        elif folderid:
            folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
            exporter = folder.OpenProperty(PR_CONTENTS_SYNCHRONIZER, IID_IExchangeExportChanges, 0, 0)
        else:
            props = self.store.GetProps([PR_IPM_SUBTREE_ENTRYID], 0)
            folder = self.store.OpenEntry(props[0].Value, None, MAPI_MODIFY)
            exporter = folder.OpenProperty(PR_HIERARCHY_SYNCHRONIZER, IID_IExchangeExportChanges, 0, 0)
        return exporter
    
    def getContentsImporter(self, folderid=None, folder=None):
        if folder:
            importer = folder.OpenProperty(PR_COLLECTOR, IID_IExchangeImportContentsChanges, 0, 0)
            proxy = ZarafaStoreContentsImporter(folder)
        elif folderid:
            folder = self.store.OpenEntry(folderid, None, MAPI_MODIFY)
            importer = folder.OpenProperty(PR_COLLECTOR, IID_IExchangeImportContentsChanges, 0, 0)
            proxy = ZarafaStoreContentsImporter(folder)
        else:
            props = self.store.GetProps([PR_IPM_SUBTREE_ENTRYID], 0)
            folder = self.store.OpenEntry(props[0].Value, None, MAPI_MODIFY)
            importer = folder.OpenProperty(PR_COLLECTOR, IID_IExchangeImportHierarchyChanges, 0, 0)
            proxy = ZarafaStoreHierarchyImporter(folder)
        proxy.importer = importer
        proxy.backend = self
        proxy.store = self.store
        proxy.addressbook = self.addressbook
        proxy.session = self.session
        return proxy
    
    def getChangeAdvisor(self):
        ecs = self.store.QueryInterface(IID_IMsgStore)
        return ecs.OpenProperty(PR_EC_CHANGE_ADVISOR, IID_IECChangeAdvisor, 0, 0)

        